 // Model railroad traverser control code for Arduino
 // With control buttons
 // With acceleration and deceleration and Emergency stop function
 // (c) Jiri Zlamal, jiri@zlamal.cz
 // January 2021
 
 #include <AccelStepper.h>                          // Including the Accellstepper library
 #include <NmraDcc.h>                               // Including the NMRA library
 
                                                    // Definition of input/output pins used:
 const int DIR_PIN = 2;                             // Direction pin for A4988 driver
 const int STEP_PIN = 3;                            // Step control pin for A4988 driver
 const int MS1_PIN = 4;                             // Microstepping control pin for A4988 driver, to the MS1 input
 const int MS2_PIN = 5;                             // Microstepping control pin for A4988 driver, to the MS3 input
 const int MS3_PIN = 6;                             // Microstepping control pin for A4988 driver, to the MS3 input
 const int ENABLE_PIN = 7;                          // Enable pin for A4988 driver, switches motor off if not moving
 const int BUTTON_PIN[4] = {A0, A1, A2, A3};        // Buttons: POS0, POS1, POS2, HOME (Pos 0 is the nearest to the Home)
 const int SENSOR_PIN = 8;                          // Limit switch - HOME position, active in HIGH state
 const int STOP_PIN = 9;                            // Emergency switch - stops the motor if connected to GND (low)
 const int POS[3] = {1000, 11133, 21266};           // Traverser positions, to be calculated manually
 const int MAX_SPEED = 2400;                        // Sets the standard full speed of the traverser
 const int SPEED = 1200;                            // Sets the speed for slow approach to the sensor
 const int ACCELERATION = 800;                      // Sets the acceleration in steps/second^2
 int MS_MODE = 8;                                   //Default microstepping mode, fot future development
 
 #define motorInterfaceType 1                       // Definition of how is the motor controlled, must be "1" if a driver is used

                                                    // Setup the AccelStepper object for the A4988 Stepper Motor Driver
 AccelStepper stepper = AccelStepper(motorInterfaceType, STEP_PIN, DIR_PIN);

 void setup() { 
 stepper.setMaxSpeed(MAX_SPEED);                    // Setting the parameters of stepper motor driver
 stepper.setAcceleration(ACCELERATION);
 
 Serial.begin(9600);                                // Enable serial communication for debugging
 Serial.println("Begin setup...");
 pinMode(SENSOR_PIN, INPUT);                        // Set up the digital input and output pins
 pinMode(ENABLE_PIN, OUTPUT);
 pinMode(STEP_PIN, OUTPUT);
 pinMode(DIR_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(MS1_PIN, OUTPUT);
 pinMode(STOP_PIN, INPUT_PULLUP);                  
  
 for (int i=0; i<4; i++) {                          // Set up the analogue push button pins (1 - 4) using pullup resistors
 pinMode(BUTTON_PIN[i], INPUT_PULLUP);
 }                                                  // End of the pushbutton loop

 
 moveToHome();                                      // Moves the traverser to the home position after rebooting/powering up.
                                                    // Comment this function if not required
 }                                                  // End of the Setup void


                                                    // Main program loop
 
void loop() {
                                                    // Poll for button presses. If a button is pressed, go to that track
                                                    // or go home if the home button is pressed

 int buttonNumber = pollButtons();                  // returns the button number (0-3)
 if (buttonNumber == 3) {                           // HOME button was pressed
 Serial.println("Moving to HOME");
 moveToHome();                                      // Move to the HOME position.
 }
 else if (buttonNumber >= 0) {                      // Track position button was pressed
 Serial.print("Moving to position ");
 Serial.println(buttonNumber);
 moveToPosition(buttonNumber);                      // Move to the selected track.
 }
 }


 
 int pollButtons() {                                // Function by Michael Hardwick
                                                    
 for (int i=0; i<4; i++) {                          // Cycle through the buttons
 if (!digitalRead(BUTTON_PIN[i])) {                 // Look closer if the button appears pushed
 int counter = 0;                                   // If yes, set a counter to zero
 while (!digitalRead(BUTTON_PIN[i])) {              // Check the button every 10 ms for 100 ms
 if (counter == 10){                                // If the button was checked 10 times, it is pressed
 return i;                                          // So return the button index
 }
 counter++;                                         // Increment the counter  how many times was the button checked
 delay(10);                                         // Delay 10 milliseconds
 }
 }
 }                                                  // End of button loop
 return -1;                                         // No button was pressed, so return -1. Necessary for correct operation!
 }



 void moveToHome() {
                                                    // This routine moves the traverser to the home position in two phases.
 digitalWrite (ENABLE_PIN, LOW);
 if(!digitalRead(SENSOR_PIN)) {                     // If the limit sensor is not active (i.e. the traverser is not home...
 stepper.moveTo (-25000);                           // Move to a distant position behind the sensor
 while(!digitalRead(SENSOR_PIN)) stepper.run();     // If the sensor is reached...
 stepper.stop ();                                   // stop the traverser...
 stepper.setCurrentPosition(0);                     // and reset the position to zero.
 }
 stepper.moveTo (400);                              // Move a bit back... 
 stepper.runToPosition();
 stepper.moveTo (-800);                             // and go again towards the sensor
 while(!digitalRead(SENSOR_PIN)) stepper.run();     // If the sensor is reached...
 stepper.stop ();                                   // stop the traverser...
 stepper.setCurrentPosition(0);                     // and reset the position to zero.
 digitalWrite (ENABLE_PIN, HIGH);
 Serial.println("Traverser is at HOME position.");  // Print a message
  }



 void moveToPosition(int buttonNumber) {            // This routine moves the traverser into the selected position
 
   Serial.print("Traverser moves to POS ");         // Print where we are giong
   Serial.println(buttonNumber);                    // The position i defined by the button pressed 
    digitalWrite (ENABLE_PIN, LOW);                 // Enable the motor driver 
    stepper.moveTo (POS[buttonNumber]);             // Run to the desired position
    while (stepper.distanceToGo () != 0 ) {         // Until the final position is reached...
       if (!digitalRead(STOP_PIN)) {                // check for the Emergency Stop signal (button or command) after each step
          Serial.println("EMERGENCY STOP");         // If the STOP function is active print a message
          stepper.stop ();                          // Stop the motor...
          stepper.setCurrentPosition(stepper.currentPosition ());  // immediately by setting the current position as a target position
          }                     
       stepper.run ();                              // If the Emergency stop is not active, run normally to the desired position
    }
    digitalWrite (ENABLE_PIN, HIGH);                // When stopped (either way), switch the motor driver off.
    
}
